@@ -243,8 +243,15 @@ def AskYesNoCancel(question, default = 0, yes=None, no=None, cancel=None, id=262
 
 
 
+# The deprecated Carbon QuickDraw APIs are no longer available as of
+# OS X 10.8.  Raise an ImportError here in that case so that callers
+# of EasyDialogs, like BuildApplet, will do the right thing.
+
+try:
+    screenbounds = Qd.GetQDGlobalsScreenBits().bounds
+except AttributeError:
+    raise ImportError("QuickDraw APIs not available")
 
-screenbounds = Qd.GetQDGlobalsScreenBits().bounds
 screenbounds = screenbounds[0]+4, screenbounds[1]+4, \
     screenbounds[2]-4, screenbounds[3]-4
 
@@ -580,9 +587,9 @@ def _process_Nav_args(dftflags, **args):
         if args[k] is None:
             del args[k]
     # Set some defaults, and modify some arguments
-    if not args.has_key('dialogOptionFlags'):
+    if 'dialogOptionFlags' not in args:
         args['dialogOptionFlags'] = dftflags
-    if args.has_key('defaultLocation') and \
+    if 'defaultLocation' in args and \
             not isinstance(args['defaultLocation'], Carbon.AE.AEDesc):
         defaultLocation = args['defaultLocation']
         if isinstance(defaultLocation, Carbon.File.FSSpec):
@@ -593,7 +600,7 @@ def _process_Nav_args(dftflags, **args):
                 defaultLocation = Carbon.File.FSRef(defaultLocation)
             args['defaultLocation'] = Carbon.AE.AECreateDesc(
                     Carbon.AppleEvents.typeFSRef, defaultLocation.data)
-    if args.has_key('typeList') and not isinstance(args['typeList'], Carbon.Res.ResourceType):
+    if 'typeList' in args and not isinstance(args['typeList'], Carbon.Res.ResourceType):
         typeList = args['typeList'][:]
         # Workaround for OSX typeless files:
         if 'TEXT' in typeList and not '\0\0\0\0' in typeList:
@@ -603,7 +610,7 @@ def _process_Nav_args(dftflags, **args):
             data = data+type
         args['typeList'] = Carbon.Res.Handle(data)
     tpwanted = str
-    if args.has_key('wanted'):
+    if 'wanted' in args:
         tpwanted = args['wanted']
         del args['wanted']
     return args, tpwanted
@@ -721,16 +728,13 @@ def AskFileForSave(
     if issubclass(tpwanted, Carbon.File.FSSpec):
         return tpwanted(rr.selection[0])
     if issubclass(tpwanted, (str, unicode)):
-        if sys.platform == 'mac':
-            fullpath = rr.selection[0].as_pathname()
-        else:
-            # This is gross, and probably incorrect too
-            vrefnum, dirid, name = rr.selection[0].as_tuple()
-            pardir_fss = Carbon.File.FSSpec((vrefnum, dirid, ''))
-            pardir_fsr = Carbon.File.FSRef(pardir_fss)
-            pardir_path = pardir_fsr.FSRefMakePath()  # This is utf-8
-            name_utf8 = unicode(name, 'macroman').encode('utf8')
-            fullpath = os.path.join(pardir_path, name_utf8)
+        # This is gross, and probably incorrect too
+        vrefnum, dirid, name = rr.selection[0].as_tuple()
+        pardir_fss = Carbon.File.FSSpec((vrefnum, dirid, ''))
+        pardir_fsr = Carbon.File.FSRef(pardir_fss)
+        pardir_path = pardir_fsr.FSRefMakePath()  # This is utf-8
+        name_utf8 = unicode(name, 'macroman').encode('utf8')
+        fullpath = os.path.join(pardir_path, name_utf8)
         if issubclass(tpwanted, unicode):
             return unicode(fullpath, 'utf8')
         return tpwanted(fullpath)
