@@ -10,19 +10,17 @@ import unittest
 class TestGB2312Map(test_multibytecodec_support.TestBase_Mapping,
                    unittest.TestCase):
     encoding = 'gb2312'
-    mapfileurl = 'http://people.freebsd.org/~perky/i18n/EUC-CN.TXT'
+    mapfileurl = 'http://www.pythontest.net/unicode/EUC-CN.TXT'
 
 class TestGBKMap(test_multibytecodec_support.TestBase_Mapping,
                    unittest.TestCase):
     encoding = 'gbk'
-    mapfileurl = 'http://www.unicode.org/Public/MAPPINGS/VENDORS/' \
-                 'MICSFT/WINDOWS/CP936.TXT'
+    mapfileurl = 'http://www.pythontest.net/unicode/CP936.TXT'
 
 class TestGB18030Map(test_multibytecodec_support.TestBase_Mapping,
                      unittest.TestCase):
     encoding = 'gb18030'
-    mapfileurl = 'http://source.icu-project.org/repos/icu/data/' \
-                 'trunk/charset/data/xml/gb-18030-2000.xml'
+    mapfileurl = 'http://www.pythontest.net/unicode/gb-18030-2000.xml'
 
 
 def test_main():
